import os

import numpy as np
from sklearn.model_selection import train_test_split
from torch.utils.data import Subset
from torchvision import datasets
from torchvision.transforms import transforms

DS_PATH = os.environ.get('DATA', '/app/data/natural_images')


def load_dataset():
    da_prepro = transforms.Compose([transforms.Resize(224),
                                    transforms.RandomCrop(224),
                                    transforms.RandomHorizontalFlip(),
                                    transforms.RandomRotation(15),
                                    transforms.ToTensor(),
                                    transforms.Normalize((0.4914, 0.4822, 0.4465),
                                                         (0.2023, 0.1994, 0.2010))])
    prepro = transforms.Compose([transforms.Resize(224),
                                 transforms.CenterCrop(224),
                                 transforms.ToTensor(),
                                 transforms.Normalize((0.4914, 0.4822, 0.4465),
                                                      (0.2023, 0.1994, 0.2010))])

    raw_da_ds = datasets.ImageFolder(DS_PATH, transform=da_prepro)
    raw_ds = datasets.ImageFolder(DS_PATH, transform=prepro)
    n_samples = len(raw_ds)
    train_idx, test_idx = train_test_split(np.arange(n_samples), test_size=0.2)
    train_idx, val_idx = train_test_split(train_idx, test_size=0.2)
    return (Subset(raw_da_ds, train_idx),
            Subset(raw_ds, val_idx),
            Subset(raw_ds, test_idx))
